/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.ereigniskal.ereigniskal;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.StringTokenizer;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataState;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.ObjectSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.Ereignis;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.EreignisTyp;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.ListeZustandsWechsel;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.SenderEreignis;
import de.bsvrz.vew.ereigniskal.ereigniskal.erinnerungsfunktion.ErinnerungsFunktionJahr;
import de.bsvrz.vew.ereigniskal.ereigniskal.senderanfrage.SenderAnfrage;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.DefinierterEintrag;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.LogischeVerknuepfung;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

/**
 * 
 * Hauptklasse der Applikation EreignisKalender.<br>
 * <br>
 * Der Ereigniskalender dient zur Verwaltung von Ereignissen und enthlt folgende Funktionen<br>
 * - Verwalten von Ereignissen<br>
 * - Automatisches Versenden von Ereignissen bei Gltigkeit/nderung der Gltigkeit<br>
 * - Abfrage von Ereignissen (auf Anfrage)<br>
 * 
 * In dem Ereigniskalender werden alle aktuellen und zuknftigen Ereignisse eingetragen und verwaltet. Das Ereignis
 * enthlt eine Kennung ber seine zeitliche Gltigkeit (Ereigniszeitbezug) sowie eine rumliche Kennung
 * (Ereignisortsbezug), fr welchen Streckenabschnitt oder welches (Teil-)Netz es relevant ist. Ereignisse werden i. d.
 * R. ber das Segment Bedienung und Visualisierung eingegeben. Welche Informationen zu einem Ereignis durch die
 * Konfiguration oder Parametrierung vorgehalten werden mssen, wird im [DatK] detailliert beschrieben. Der
 * Ereigniskalender enthlt alle Ereignisse, die im Netz aktuell oder zuknftig vorliegen werden.<br>
 * 
 * <br>
 * Folgende Startparameter sind mglich:<br>
 * <br>
 * DAF-Startparameter wie in SSB beschrieben und durch StandardApplication verarbeitet:<br>
 * <br>
 * Beispiele:<br>
 * -datenverteiler=rechnerDav:8083<br>
 * -debugLevelStdErrText=CONFIG<br>
 * -debugLevelFileText=CONFIG<br>
 * -benutzer=Tester<br>
 * -authentifizierung=C:\kernsoftware-3.4.2\passwd<br>
 * 
 * <br>
 * Spezielle Startparameter:<br>
 * <br>
 * <ul>
 * <li>-kalender= hier wird die Pid des Konfigurationsobektes uebergeben, welches von typ.kalender abgeleitet ist</li>
 * <br>
 * <li>-neuberechnung= hier wird der Zeitpunkt als String bergeben, bei welcher eine Neuberechnung der Ereigniszustnde
 * fr das nchste Jahr erfolgt</li>
 * <br>
 * 
 * <br>
 * Beispiele:<br>
 * -kalender=kv.aoe.dambach.test1<br>
 * -neuberechnung="01.12.2008 00:00:00,000"<br>
 * 
 * 
 * @version $Revision: 1.10 $ / $Date: 2015/10/12 13:13:15 $ / ($Author: Pittner $)
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 */
/**
 * @author Pittner
 *
 */
public class EreignisKalender implements StandardApplication, ClientSenderInterface, ClientReceiverInterface,
    MutableSetChangeListener
{
  /**
   * DebugLogger fr Debug-Ausgaben
   */
  private static Debug _debug;

  /**
   * -datenverteiler von Kommandozeile
   */
  private String _dav;

  /**
   * Verbindung zum Datenverteiler
   */
  private static ClientDavInterface _connection;

  /**
   * String fuer den Kalender
   */
  private String _kalender;

  /**
   * String fuer den Kalender
   */
  private String _neuBerechnung;

  /**
   * String fuer die Menge der SystemKalenderEintraege
   */
  private String _systemKalenderEintraege;

  /**
   * String fuer die Menge der Ereignisse
   */
  private String _ereignisse;

  /**
   * String fuer die Menge der EreignisTypen
   */
  private String _ereignisTypen;

  /**
   * String fuer eine Attributgruppe
   */
  private String OBJECT_ATG;

  /**
   * String einen Aspekt
   */
  private String OBJECT_ASP;

  /**
   * Attributgruppe
   */
  private AttributeGroup _attributgruppe;

  /**
   * Aspekt
   */
  private Aspect _aspekt;

  /**
   * Datenbeschreibung
   */
  private DataDescription _datenbeschreibung;

  /**
   * Datenmodell
   */
  private DataModel _datenmodell;

  /**
   * Simulationsvariante
   */
  private short _simulationsvariante;

  /**
   * Senderrolle
   */
  private SenderRole _senderrolle;

  /**
   * Empfaengeroption
   */
  private ReceiveOptions _empfaengeroptionen;

  /**
   * Empfaengerrrolle
   */
  private ReceiverRole _empfaengerrolle;

  /**
   * Klasse fr die Zeitstempelliste
   */
  private ListeZustandsWechsel lzwZ = new ListeZustandsWechsel();

  /**
   * Klasse fr die Zeitstempelliste
   */
  private ListeZustandsWechsel lzwV = new ListeZustandsWechsel();

  /**
   * Die statische Liste der SystemKalenderEintraege
   * 
   * @uml.property name="skeList"
   * @uml.associationEnd qualifier="key:java.lang.Object de.bsvrz.vew.ereigniskal.ereigniskal.SystemkalenderEintrag"
   */
  private static Map<String, SystemkalenderEintrag> skeList = new HashMap<String, SystemkalenderEintrag>();

  /**
   * Die statische Liste der Ereignisse
   * 
   * @uml.property name="evtList"
   * @uml.associationEnd qualifier="key:java.lang.Object de.bsvrz.vew.ereigniskal.ereigniskal.Ereignis"
   */
  private static Map<String, Ereignis> evtList = new HashMap<String, Ereignis>();

  /**
   * Die statische Liste der EreignisTypen
   * 
   */
  private static Map<String, EreignisTyp> evtTypList = new HashMap<String, EreignisTyp>();

  /**
   * Die statische Liste der Gueltigkeiten
   * 
   */
  private static Map<String, Boolean[]> gueltigkeitList = new HashMap<String, Boolean[]>();

  /**
   * Zaehler fuer SystemKalenderEintraege
   */
  private int cntSke;

  /**
   * Zaehler fuer Ereignisse
   */
  private int cntEvt;

  /**
   * Zaehler fuer geparste Ske
   */
  private int cntParse;

  /**
   * Zaehler fuer geparste EreignisTypen
   */
  private int cntEvtTyp;

  /**
   * Liste mit den Pid's der SystemKalenderEintraege
   * 
   * @uml.property name="parseList"
   * @uml.associationEnd qualifier="key:java.lang.Object java.lang.String"
   */
  private Map<String, String[]> parseList = new HashMap<String, String[]>();

  private Boolean _inInit;

  /**
   * Konfigurationsobjekt
   */
  private ConfigurationObject _configObj;

  /**
   * Konstruktor
   */
  public EreignisKalender()
  {

    try
    {
      Thread.sleep(100);
    }
    catch (InterruptedException e)
    {
      e.printStackTrace();
    }
  }

  /**
   * berschriebene Methode von StandardApplication, die die speziellen Startparameter auswertet.<br>
   * Die Liste der Konfigurationsbereiche wird durch Aufspaltung des bergebenen Strings erstellt und die speziellen
   * Startparameter werden in die Log-Datei eingetragen.
   * 
   * @param argumentList
   *          siehe sys.funclib.application.StandardApplication#parseArguments(sys.funclib.ArgumentList)
   */
  public void parseArguments(ArgumentList argumentList) throws Exception
  {
    _debug = Debug.getLogger();

    _debug.fine("argumentList = " + argumentList);

    _dav = argumentList.fetchArgument("-datenverteiler=").asNonEmptyString();

    _debug.fine("datenverteiler = '" + _dav + "'");

    _kalender = argumentList.fetchArgument("-kalender=:aoe:").asNonEmptyString();

    if (_kalender.equals(":aoe:"))
    {
      _kalender = null;
    }

    _neuBerechnung = argumentList.fetchArgument("-neuberechnung=").asString();

    _systemKalenderEintraege = argumentList.fetchArgument("-systemKalenderEintraege=SystemKalenderEintrge")
        .asNonEmptyString();

    _ereignisse = argumentList.fetchArgument("-ereignisse=Ereignisse").asNonEmptyString();

    _ereignisTypen = argumentList.fetchArgument("-ereignisTypen=EreignisTypen").asNonEmptyString();

    argumentList.fetchUnusedArguments();

  }

  /**
   * berschriebene Methode von StandardApplication, die die Initialisierung durchfhrt.<br>
   * Entsprechend dem Argument -layer wird die entsprechende Methode aufgerufen und danach die Log-Datei geschlossen.<br>
   * 
   * @param connection
   *          siehe sys.funclib.application.StandardApplication#initialize(stauma.dav.clientside.ClientDavInterface)
   */
  public void initialize(ClientDavInterface connection) throws Exception
  {

    try
    {
      _inInit = true;

      _connection = connection;

      _datenmodell = _connection.getDataModel();

      _senderrolle = SenderRole.source(); // Rolle der Applikation

      _empfaengeroptionen = ReceiveOptions.normal();

      _empfaengerrolle = ReceiverRole.receiver();

      // Hole SystemKalenderEintraege
      OBJECT_ATG = "atg.systemKalenderEintrag";
      OBJECT_ASP = "asp.parameterSoll";

      _attributgruppe = _connection.getDataModel().getAttributeGroup(OBJECT_ATG);
      _aspekt = _connection.getDataModel().getAspect(OBJECT_ASP);
      _simulationsvariante = 0;
      _datenbeschreibung = new DataDescription(_attributgruppe, _aspekt, _simulationsvariante);

      if (_kalender != null)
      {
        _configObj = (ConfigurationObject)_datenmodell.getObject(_kalender);
      }
      else
      {
        _configObj = (ConfigurationObject)_connection.getLocalConfigurationAuthority();
        _kalender = _configObj.getPid();
      }

      if (_configObj == null || !_configObj.isOfType("typ.kalender"))
      {
        final String errorMessage = "Es gibt kein passendes Kalender-Objekt (typ.kalender). Applikation kann nicht gestartet werden.";
        // _debug.error(errorMessage);
        _connection.disconnect(true, errorMessage);
        this.wait(1000);
        System.exit(0);
        // throw new ExceptionInInitializerError(errorMessage);
      }

      List objSke = readSystemKalenderEintragMenge();

      cntSke = 0;

      _debug.fine("Anmeldung SystemKalenderEintrge");

      subscribeReceiver(objSke);

      for (Object object : objSke)
      {
        SystemObject so = (SystemObject)object;

      }

      synchronized (this)
      {

        while ((cntSke < objSke.size()))
        {

          this.wait();
        }
      }

      _debug.fine("Alle SystemKalenderEintraege empfangen!");

      parseArbeiterSke();

      // if (_kalender != null){
      // _configObj = (ConfigurationObject)_datenmodell.getObject(_kalender);
      // }else{
      // _configObj = (ConfigurationObject)_connection.getLocalConfigurationAuthority();
      // _kalender = _configObj.getPid();
      // }

      MutableSet ms = _configObj.getMutableSet("SystemKalenderEintrge");

      if (ms != null)
      {
        _debug.config("Listener Menge SystemKalenderEintrge angemeldet");

        ms.addChangeListener(this);
      }
      else
        _debug.error("Menge SystemKalenderEintrge ist null");

      // Hole EreignisTypParameter
      OBJECT_ATG = "atg.ereignisTypParameter";
      OBJECT_ASP = "asp.parameterSoll";

      _attributgruppe = _connection.getDataModel().getAttributeGroup(OBJECT_ATG);
      _aspekt = _connection.getDataModel().getAspect(OBJECT_ASP);
      _simulationsvariante = 0;
      _datenbeschreibung = new DataDescription(_attributgruppe, _aspekt, _simulationsvariante);

      List objEvtTyp = readEreignisTypMenge();

      cntEvtTyp = 0;

      _debug.fine("Anmeldung EreignisTypen");

      subscribeReceiver(objEvtTyp);

      synchronized (this)
      {

        while (cntEvtTyp < (objEvtTyp.size()))
        {
          this.wait();
        }
      }

      _debug.fine("Alle EreignisTypen erhalten");

      // Hole EreignisParameter
      OBJECT_ATG = "atg.ereignisParameter";
      OBJECT_ASP = "asp.parameterSoll";

      _attributgruppe = _connection.getDataModel().getAttributeGroup(OBJECT_ATG);
      _aspekt = _connection.getDataModel().getAspect(OBJECT_ASP);
      _simulationsvariante = 0;
      _datenbeschreibung = new DataDescription(_attributgruppe, _aspekt, _simulationsvariante);

      List objEvt = readEreignisMenge();

      cntEvt = 0;

      subscribeReceiver(objEvt);

      synchronized (this)
      {

        while (cntEvt < (objEvt.size()))
        {

          this.wait();

        }
      }

      _debug.fine("Alle Ereignisse empfangen!");

      ms = _configObj.getMutableSet("Ereignisse");

      if (ms != null)
      {
        _debug.config("Listener Menge Ereignisse angemeldet");

        ms.addChangeListener(this);
      }
      else
        _debug.error("Menge Ereignisse ist null");

      // Alle Ereignistimer werden gestellt
      // for (Ereignis e : evtList.values())
      // {
      //
      // e.setErinnerungsFunktion(true);
      //
      // }
      //
      // _debug.config("Alle Ereignisse Scheduled!");

      AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAnfrage");
      Aspect asp = _connection.getDataModel().getAspect("asp.anfrage");
      DataDescription dd = new DataDescription(atg, asp);

      SystemObject kalender = _connection.getDataModel().getObject(_kalender);

      _connection.subscribeReceiver(this, kalender, dd, ReceiveOptions.normal(), ReceiverRole.drain());

      _debug.config("Anmeldung als Senke " + kalender.getPid());

      SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
      Date jetzt = new Date();

      try
      {
        Date neu = df.parse(_neuBerechnung);

        if (neu.getTime() > jetzt.getTime())
        {
          ErinnerungsFunktionJahr efj = new ErinnerungsFunktionJahr(new NeuBerechnung(evtList, neu.getTime()));
          efj.load(neu.getTime());
        }
        else
        {
          _debug.error("Fehler Zeitpunkt Neuberechung, verwende: " + jetzt);
          ErinnerungsFunktionJahr efj = new ErinnerungsFunktionJahr(new NeuBerechnung(evtList, jetzt.getTime()));
          efj.load(jetzt.getTime());

        }
      }
      catch (ParseException e1)
      {
        // TODO Auto-generated catch block
        _debug.error("Fehler Zeitpunkt -neuberechung verwende: " + jetzt);
        ErinnerungsFunktionJahr efj = new ErinnerungsFunktionJahr(new NeuBerechnung(evtList, jetzt.getTime()));
        efj.load(jetzt.getTime());
      }

      _inInit = false;

      _debug.config("Ereigniskalender bereit");

    }
    catch (Exception e)
    {

      e.printStackTrace();
      _debug.error(e.getMessage());

    }

  }// public void initialize(ClientDavInterface connection)

  /**
   * Liest die Menge der Systemkalendereintrge
   * 
   * @return Liste der Systemkalendereintrge
   * 
   * @throws Exception
   */
  public List readSystemKalenderEintragMenge() throws Exception
  {
    // ConfigurationObject kalender = (ConfigurationObject)_connection.getDataModel().getObject("kalender.kalender");
    ConfigurationObject kalender = (ConfigurationObject)_connection.getDataModel().getObject(_kalender);

    ObjectSet objekte = kalender.getObjectSet(_systemKalenderEintraege);

    List<SystemObject> list = objekte.getElements();

    List<SystemObject> tmp = new ArrayList<SystemObject>();

    tmp.addAll(list);

    for (SystemObject systemObject : list)
    {
      if (!systemObject.isValid())
      {
        _debug.fine("ungueltiges Objekt: " + systemObject);
        tmp.remove(systemObject);
      }
    }

    return tmp;
  }

  /**
   * Liest die Menge der Ereignisse
   * 
   * @return Liste der Ereignisse
   * 
   * @throws Exception
   */
  public List readEreignisMenge() throws Exception
  {
    ConfigurationObject kalender = (ConfigurationObject)_connection.getDataModel().getObject(_kalender);

    ObjectSet objekte = kalender.getObjectSet(_ereignisse);

    List<SystemObject> list = objekte.getElements();

    List<SystemObject> tmp = new ArrayList<SystemObject>();

    tmp.addAll(list);

    for (SystemObject systemObject : list)
    {
      if (!systemObject.isValid())
      {
        _debug.fine("ungueltiges Objekt: " + systemObject);
        tmp.remove(systemObject);
      }
    }

    return tmp;
  }

  /**
   * Liest die Menge der Ereignistypen
   * 
   * @return Liste der Ereignistypen
   * 
   * @throws Exception
   */
  public List readEreignisTypMenge() throws Exception
  {
    ConfigurationObject kalender = (ConfigurationObject)_connection.getDataModel().getObject(_kalender);

    ObjectSet objekte = kalender.getObjectSet(_ereignisTypen);

    List<SystemObject> list = objekte.getElements();

    List<SystemObject> tmp = new ArrayList<SystemObject>();

    tmp.addAll(list);

    for (SystemObject systemObject : list)
    {
      if (!systemObject.isValid())
      {
        _debug.fine("ungueltiges Objekt: " + systemObject);
        tmp.remove(systemObject);
      }
    }

    return tmp;
  }

  /** Anmeldung zum Senden von Daten */
  public void subscribeSender(List objlist)
  {
    // Anmelden
    try
    {
      _connection.subscribeSender(EreignisKalender.this, objlist, _datenbeschreibung, _senderrolle);
    }
    catch (OneSubscriptionPerSendData e)
    {
      _debug.fine("Datenidentifikation ist bereits angemeldet.");
    }
  }

  /** Anmeldung zum Empfangen von Daten */
  public void subscribeReceiver(List objlist)
  {
    // Anmelden
    _connection.subscribeReceiver(EreignisKalender.this, objlist, _datenbeschreibung, _empfaengeroptionen,
        _empfaengerrolle);

  }

  /** Abmeldung vom Senden der Daten */
  public void unsubscribeSender(List objlist)
  {
    // Abmelden
    _connection.unsubscribeSender(EreignisKalender.this, objlist, _datenbeschreibung);
    _debug.fine("unsubscribe.");
  }

  /** Abmeldung vom Empfangen der Daten */
  public void unsubscribeReceiver(List objlist)
  {
    // Abmelden
    _connection.unsubscribeReceiver(EreignisKalender.this, objlist, _datenbeschreibung);
    _debug.fine("unsubscribe.");
  }

  /** Senden von Daten als Quelle */
  public void send(SystemObject sysobj)
  {
    // Zeitstempel
    long zeitstempel = System.currentTimeMillis();

    // AttributeGroup atg =
    AttributeGroup atg = _connection.getDataModel().getAttributeGroup(OBJECT_ATG);
    // Datensatz erstellen
    Data datensatz = _connection.createData(atg);

    // Telegramm erstellen
    ResultData datentelegramm = new ResultData(sysobj, _datenbeschreibung, zeitstempel, datensatz);
    // Versenden
    try
    {
      _connection.sendData(datentelegramm);
    }
    catch (Exception e)
    {
      _debug.error("Fehler: " + e.getMessage());
    }
  }

  /*
   * (non-Javadoc)
   * 
   * @see stauma.dav.clientside.ClientReceiverInterface#update(stauma.dav.clientside.ResultData[])
   */
  public void update(ResultData[] resultData)
  {

    for (ResultData data : resultData)
    {

      if (data.getDataState() == DataState.NO_SOURCE)
        synchronized (this)
        {
          notify();
        }

      if (data.getObject().isOfType("typ.systemKalenderEintrag"))
      {

        if (data.getData() != null)
        {

          String s = data.getData().getItem("Definition").valueToString();

          int pos = s.indexOf(":=");

          String[] str = new String[2];
          if (pos != -1)
          {

            str[0] = s.substring(0, pos);
            str[1] = s.substring(pos + 2);

          }
          else
          {
            str[0] = s;
            str[1] = s;

          }

          String skePid = data.getObject().getPid().toLowerCase();

          parseList.put(skePid, str);

          if (!_inInit)
          {

            _debug.fine("Starte Parser!!!");
            parseArbeiterSke();

            if (skeList.containsKey(skePid.toLowerCase()))
            {

              ArrayList<String> tmp = new ArrayList<String>();

              holeSkeRekursiv(skePid, tmp);

              for (String x : tmp)
              {

                Ereignis ex = evtList.get(x);

                if (!_inInit)
                {
                  SystemkalenderEintrag ske = skeList.get(skePid);

                  ex.getListeZustandsWechselZeit().setListeZustandsWechsel(ske.getListeZustandsWechsel());

                  if (ex.getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
                  {

                    lzwV = new ListeZustandsWechsel();

                    int len = ex.lenVerkehrlicheGueltigkeit;

                    ex.setLaengeVerkehrlicheGueltigkeit(len, false);

                    for (int j = 0; j < len; j++)
                    {

                      lzwV.setGueltigkeitVerkehr(ex.getListeZustandsWechselZeit().getListeZustandsWechsel(), ex.anf[j],
                          ex.bezAnf[j], ex.end[j], ex.bezEnd[j]);

                      ex.setListeZustandsWechselVerkehr(lzwV);

                    }

                  }
                  else
                    _debug.fine(ex.getPid() + " hat keine zeitliche Zustandsliste");

                  long zeit = System.currentTimeMillis();
                  ResultData result = ex.berechneDatenEreignisZustand(zeit);

                  SenderEreignis sender = new SenderEreignis(_connection, SenderRole.source());
                  sender.sende(result);

                  if (ex.getErinnerungsFunktion() != null)
                    ex.getErinnerungsFunktion().unload();

                  if (evtList.containsKey(x))
                    evtList.remove(x);

                  evtList.put(x, ex);

                  ex.setErinnerungsFunktion();
                  // ex.setErinnerungsFunktion(true);
                  // ex.setErinnerungsFunktion(false);

                }

              }

            }
            else
            {
//              _debug.error("SystemKalenderEintrag: " + skePid + " ist nicht vorhanden");
            }

          }
          else
            cntSke++;

        }
        else
        {

          _debug.fine(data.getObject().getPid() + ": Parameter-Data ist null");

        }

        synchronized (this)
        {
          notify();
        }

      }
      else if (data.getObject().isOfType("typ.ereignis"))
      {

        if (data.getData() != null)
        {
          boolean aenderung = false;
          boolean ungueltig = false;

          AttributeGroup atgRcv = data.getDataDescription().getAttributeGroup();

          AttributeGroup atgSub = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");

          if (atgRcv.equals(atgSub))
          {

            String beginGueltigkeit = data.getData().getItem("BeginnZeitlicheGltigkeit").valueToString();
            String endeGueltigkeit = data.getData().getItem("EndeZeitlicheGltigkeit").valueToString();

            Date bg = new Date();
            Date eg = new Date();

            // SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");
            SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

            try
            {

              bg = df.parse(beginGueltigkeit);
              eg = df.parse(endeGueltigkeit);

            }
            catch (ParseException e)
            {
              // TODO Auto-generated catch block
              e.printStackTrace();
            }

            Ereignis evt = null;

            // Zeitangabe erfolgt ueber SystemKalenderEintrag
            if (eg.getTime() == bg.getTime())
            {

              String skeRef = data.getData().getItem("SystemKalenderEintragReferenz").asTextValue().getValueText();

              String skePid = skeRef.toLowerCase();

              if (skeList.containsKey(skePid))
              {

                evt = new Ereignis(data.getObject().getPid(), skePid);

                if (evt.getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
                {

                  // if (evt.getErinnerungsFunktion() != null)
                  // evt.getErinnerungsFunktion().unload();
                  //
                  // if (evtList.containsKey(evt.getPid()))
                  // evtList.remove(evt.getPid());
                  //
                  // evtList.put(evt.getPid(), evt);

                  aenderung = true;

                }

              }
              else
              {
//                _debug.error("SystemKalenderEintragReferenz: " + data.getObject().getPid() + " ist nicht vorhanden");
                ungueltig = true;
              }

            }
            else
            {

              evt = new Ereignis(data.getObject().getPid());

              _debug.fine(evt.getPid() + " : Definition ueber zeitliche Gueltigkeit");

              lzwZ = evt.getListeZustandsWechselZeit();
              lzwZ.setGueltigkeitZeit(bg.getTime(), eg.getTime());

              evt.setListeZustandsWechselZeit(lzwZ);

              if (evt.getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
              {

                // if (evt.getErinnerungsFunktion() != null)
                // evt.getErinnerungsFunktion().unload();
                //
                // if (evtList.containsKey(evt.getPid()))
                // evtList.remove(evt.getPid());
                //
                // evtList.put(evt.getPid(), evt);

                aenderung = true;
              }

            }// if (eg.getTime() == bg.getTime()){
            Array rg = data.getData().getArray("RumlicheGltigkeit");
            if (!ungueltig && rg.getLength() > 0)
            {

              for (int i = 0; i < rg.getLength(); i++)
              {

                String str = rg.getItem(i).asTextValue().getValueText();

                if (evt != null)
                {
                  evt.setRaeumlicheGueltigkeit(str);
                }
                else
                {

                  _debug.fine("Fehler Rumliche Gueltigkeit, Ereignis ist null!");

                }
              }
            }

            Array a = data.getData().getArray("VerkehrlicheGltigkeit");

            Long anf[] = new Long[a.getLength()];
            Long end[] = new Long[a.getLength()];
            String s1 = null;
            String s2 = null;
            Integer[] bezAnf = new Integer[a.getLength()];
            Integer[] bezEnd = new Integer[a.getLength()];

            boolean verkehrlichGueltig = false;

            if (a.getLength() > 0)
            {

              verkehrlichGueltig = true;

              for (int i = 0; i < a.getLength(); i++)
              {

                anf[i] = a.getItem(i).getTimeValue("ZeitDauerAnfangIntervall").getMillis();
                s1 = a.getItem(i).getScaledValue("ZeitBezugAnfangIntervall").getValueText();

                end[i] = a.getItem(i).getTimeValue("ZeitDauerEndeIntervall").getMillis();
                s2 = a.getItem(i).getScaledValue("ZeitBezugEndeIntervall").getValueText();

                if (s1.equalsIgnoreCase("vorAnfang"))
                {

                  bezAnf[i] = 0;

                }
                else if (s1.equalsIgnoreCase("nachAnfang"))
                {

                  bezAnf[i] = 1;

                }
                else if (s1.equalsIgnoreCase("vorEnde"))
                {

                  bezAnf[i] = 2;

                }
                else if (s1.equalsIgnoreCase("nachEnde"))
                {

                  bezAnf[i] = 3;
                }
                else
                {
                  _debug.fine("Fehler Zeitbezug Anfang!");
                }

                if (s2.equalsIgnoreCase("vorAnfang"))
                {

                  bezEnd[i] = 0;

                }
                else if (s2.equalsIgnoreCase("nachAnfang"))
                {

                  bezEnd[i] = 1;

                }
                else if (s2.equalsIgnoreCase("vorEnde"))
                {

                  bezEnd[i] = 2;

                }
                else if (s2.equalsIgnoreCase("nachEnde"))
                {

                  bezEnd[i] = 3;

                }
                else
                {

                  _debug.fine("Fehler Zeitbezug Ende!");

                }

              }

            }

            if (!ungueltig && verkehrlichGueltig)
            {
              verkehrlichGueltig = false;

              // if (evtList.containsKey(data.getObject().getPid()))
              if (true)
              {
                // evt = evtList.get(data.getObject().getPid());
                lzwV = evt.getListeZustandsWechselVerkehr();

                evt.setLaengeVerkehrlicheGueltigkeit(a.getLength(), true);

                for (int j = 0; j < a.getLength(); j++)
                {
                  if (lzwV != null)
                  {

                    ListeZustandsWechsel listeZeit = evt.getListeZustandsWechselZeit();

                    if (listeZeit != null)
                    {
                      SortedMap<Long, Boolean> map = listeZeit.getListeZustandsWechsel();

                      // nicht auf der Referenz der zeilt. Liste Liste arbeiten
                      // SortedMap<Long, Boolean> map = new TreeMap<Long, Boolean>();
                      // map.putAll(tmp);

                      if (map != null)
                      {

                        if (anf[j] != null && bezAnf[j] != null && end[j] != null && bezEnd[j] != null)
                        {
                          lzwV.setGueltigkeitVerkehr(map, anf[j], bezAnf[j], end[j], bezEnd[j]);

                          evt.setListeZustandsWechselVerkehr(lzwV);
                          evt.anf[j] = new Long(anf[j]);
                          evt.bezAnf[j] = new Integer(bezAnf[j]);
                          evt.end[j] = new Long(end[j]);
                          evt.bezEnd[j] = new Integer(bezEnd[j]);
                        }
                        else
                        {
                          _debug
                              .error(evt.getPid()
                                  + " : Konnte ZustandswechselVerkehr nicht berechnen: Definition verkehrliche Gltigkeit ist null: "
                                  + anf[j] + " - " + bezAnf[j] + " - " + end[j] + " - " + bezEnd[j]);
                        }
                      }
                      else
                      {
                        _debug.error("Konnte ZustandswechselVerkehr nicht berechnen: ListeZustandsWechsel ist null: "
                            + evt.getPid());
                      }
                    }
                    else
                    {
                      _debug.error("Konnte ZustandswechselVerkehr nicht berechnen: ListeZustandsWechselZeit ist null: "
                          + evt.getPid());
                    }
                  }
                  else
                  {
                    _debug
                        .error("Konnte ZustandswechselVerkehr nicht berechnen: ListeZustandsWechselVerkehr ist null: "
                            + evt.getPid());
                  }
                }

                // if (evt.getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
                // {
                // lzwV = evt.getListeZustandsWechselVerkehr();
                // evt.setLaengeVerkehrlicheGueltigkeit(a.getLength(), true);
                //
                // for (int j = 0; j < a.getLength(); j++)
                // {
                // lzwV.setGueltigkeitVerkehr(evt.getListeZustandsWechselZeit().getListeZustandsWechsel(), anf[j],
                // bezAnf[j], end[j], bezEnd[j]);
                // evt.setListeZustandsWechselVerkehr(lzwV);
                // evt.anf[j] = new Long(anf[j]);
                // evt.bezAnf[j] = new Integer(bezAnf[j]);
                // evt.end[j] = new Long(end[j]);
                // evt.bezEnd[j] = new Integer(bezEnd[j]);
                // }
                // }

              }
              else
                _debug.fine(data.getObject().getPid() + " nicht in evtList enthalten");
            }
            else
            {
              if (evtList.containsKey(data.getObject().getPid()))
              {
                if (evt != null)
                  evt.setListeZustandsWechselVerkehr(new ListeZustandsWechsel());
              }
            }

            if (!ungueltig && aenderung)
            {
              // if (!_inInit)
              {

                long zeit = System.currentTimeMillis();

                // ResultData result = evt.berechneDatenEreignisZustand(zeit);
                //
                // SenderEreignis sender = new SenderEreignis(_connection, SenderRole.source());
                // sender.sende(result);

                if (evt.getErinnerungsFunktion() != null)
                  evt.getErinnerungsFunktion().unload();

                if (evtList.containsKey(evt.getPid()))
                  evtList.remove(evt.getPid());

                evtList.put(evt.getPid(), evt);

                lzwV = evt.getListeZustandsWechselVerkehr();
                evt.setLaengeVerkehrlicheGueltigkeit(a.getLength(), true);

                for (int j = 0; j < a.getLength(); j++)
                {
                  if (lzwV != null)
                  {

                    ListeZustandsWechsel listeZeit = evt.getListeZustandsWechselZeit();

                    if (listeZeit != null)
                    {
                      SortedMap<Long, Boolean> map = listeZeit.getListeZustandsWechsel();

                      if (map != null)
                      {

                        if (anf[j] != null && bezAnf[j] != null && end[j] != null && bezEnd[j] != null)
                        {
                          lzwV.setGueltigkeitVerkehr(map, anf[j], bezAnf[j], end[j], bezEnd[j]);

                          evt.setListeZustandsWechselVerkehr(lzwV);
                          evt.anf[j] = new Long(anf[j]);
                          evt.bezAnf[j] = new Integer(bezAnf[j]);
                          evt.end[j] = new Long(end[j]);
                          evt.bezEnd[j] = new Integer(bezEnd[j]);
                        }
                        else
                        {
                          _debug
                              .error(evt.getPid()
                                  + " : Konnte ZustandswechselVerkehr nicht berechnen: Definition verkehrliche Gltigkeit ist null: "
                                  + anf[j] + " - " + bezAnf[j] + " - " + end[j] + " - " + bezEnd[j]);
                        }
                      }
                      else
                      {
                        _debug.error("Konnte ZustandswechselVerkehr nicht berechnen: ListeZustandsWechsel ist null: "
                            + evt.getPid());
                      }
                    }
                    else
                    {
                      _debug.error("Konnte ZustandswechselVerkehr nicht berechnen: ListeZustandsWechselZeit ist null: "
                          + evt.getPid());
                    }
                  }
                  else
                  {
                    _debug
                        .error("Konnte ZustandswechselVerkehr nicht berechnen: ListeZustandsWechselVerkehr ist null: "
                            + evt.getPid());
                  }
                }

                // if (!_inInit)
                // evt.setErinnerungsFunktion(false);
                ResultData result = evt.berechneDatenEreignisZustand(zeit);

                SenderEreignis sender = new SenderEreignis(_connection, SenderRole.source());
                sender.sende(result);

                // TODO aus Kommentar rausnehmen!!!
                evt.setErinnerungsFunktion();
              }

            }

          }
          else
          {
            _debug.fine("Fehler Attributgruppe:  " + atgRcv.getName());
          }

          cntEvt++;

        }
        else
        {
          _debug.fine(data.getObject().getPid() + ": Parameter-Data ist null");
        }

        synchronized (this)
        {
          notify();
        }

      }
      else if (data.getObject().isOfType("typ.kalender"))
      {

        if (data.getData() != null)
        {

          String von = data.getData().getItem("Anfangszeitpunkt").valueToString();
          String bis = data.getData().getItem("Endzeitpunkt").valueToString();

          SimpleDateFormat df = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");

          Date d1 = new Date();
          Date d2 = new Date();

          try
          {
            d1 = df.parse(von);
            d2 = df.parse(bis);
          }
          catch (ParseException e)
          {
            // TODO Auto-generated catch block
            e.printStackTrace();
          }

          if (d1.getTime() < d2.getTime())
          {

            SenderAnfrage sa = new SenderAnfrage(data);
            sa.send();

          }
          else
          {

            _debug.error("ungueltige Zeitabgabe bei der EreignisKalenderAnfrage!");
            SenderAnfrage sa = new SenderAnfrage(data);
            sa.sendFehler();
          }

        }
        else
        {
          _debug.fine(data.getObject().getPid() + ": Parameter-Data ist null");
        }

      }
      else if (data.getObject().isOfType("typ.ereignisTyp"))
      {
        if (data.getData() != null)
        {
          EreignisTyp evtTyp = null;

          Long prio = data.getData().getUnscaledValue("EreignisTypPrioritt").longValue();

          if (prio != null)
          {
            evtTyp = new EreignisTyp(data.getObject().getPid());

            evtTyp.setPrioritaet(prio);

            evtTypList.put(evtTyp.getPid(), evtTyp);
          }

          cntEvtTyp++;
        }
        else
        {

          _debug.fine(data.getObject().getPid() + ": Parameter-Data ist null");

        }

        synchronized (this)
        {
          notify();
        }

      }
      else
      {

        if (data.getData() != null)
        {

          EreignisKalender._debug.fine("Typ nicht versorgt: " + data.getData());

        }

      }

    }

  }


  /**
   * Holt alle Ereignisse, mit der entsprechenden SkeReferenz
   * @param skePid
   * @param tmp
   */
  public static void holeSkeRekursiv(String skePid, ArrayList<String> tmp)
  {
    ArrayList<String> list = new ArrayList<String>();

    list.add(skePid);

    for (SystemkalenderEintrag ske : skeList.values())
    {
      String definition = ske.getDefinition();

      if (ske instanceof LogischeVerknuepfung)
      {
        if (definition.contains("{") && definition.contains("}"))
        {
          String klammerAusdruck = definition.substring(definition.indexOf("{") + 1, definition.indexOf("}"));

          StringTokenizer st = new StringTokenizer(klammerAusdruck, ",");
         
          while (st.hasMoreTokens())
          {
            String s = st.nextToken();

            if (s.contains("nicht"))
            {
              s = s.replace("nicht", "");
              s = s.trim();
            }
            SystemkalenderEintrag ref = skeList.get(skePid);

            if (s.equalsIgnoreCase(ref.getName()))
            {
              if (!list.contains(ske.getPid()))
              {
                list.add(ske.getPid());
              }
            }
          }
        }
      }
    }

    for (Ereignis e : evtList.values())
    {
      String ref = e.getSkeReferenz();

      if (ref != null)
      {

        for (String string : list)
        {
          if (ref.equals(string))
          {
            _debug.fine("Ereignis-> " + e.getPid() + " trgt Referenz auf: " + skePid);
            if (e.getListeZustandsWechselZeit().getListeZustandsWechsel() != null)
            {
              tmp.add(e.getPid());
            }
          }
        }
      }
    }
  }
  
  /**
   * Holt die einzelnen Ske die in dem bergebenen Eintrag definiert sind 
   * @param skePid
   * @param tmp
   */
  public static void holeSkeRekursivNeu(String skePid, HashMap<String, Boolean> tmp)
  {
    if(skeList.containsKey(skePid)){
      SystemkalenderEintrag ske = skeList.get(skePid);
//      if (ske instanceof LogischeVerknuepfung ||  ske instanceof DefinierterEintrag)
        if (ske instanceof LogischeVerknuepfung)
      {
        
        String definition = ske.getDefinition();
        if (definition.contains("{") && definition.contains("}"))
        {
          String klammerAusdruck = definition.substring(definition.indexOf("{") + 1, definition.indexOf("}"));
          
          StringTokenizer st = new StringTokenizer(klammerAusdruck, ",");
          
          while (st.hasMoreTokens())
          {
            
            
            String s = st.nextToken();
            if (s.contains("UND"))
              break;
            
            boolean nicht = true;
            if (s.contains("nicht"))
            {
              s = s.replace("nicht", "");
              s = s.trim();
              nicht = false;
            }
            
            
            for (SystemkalenderEintrag skeTmp : skeList.values())
            {
              if (s.equalsIgnoreCase(skeTmp.getName()))
              {
                skePid = skeTmp.getPid();
                break;
              }
            }
            
            if (!tmp.containsKey(skePid)){
              tmp.put(skePid, nicht);
              holeSkeRekursivNeu(skePid, tmp);                
            }
          }
        }
      }else{                        
          if (!tmp.containsKey(skePid))
            tmp.put(ske.getPid(), true);
      }
    }          
  }

  /*
   * (non-Javadoc)
   * 
   * @see
   * stauma.dav.configuration.interfaces.MutableSetChangeListener#update(stauma.dav.configuration.interfaces.MutableSet,
   * stauma.dav.configuration.interfaces.SystemObject[], stauma.dav.configuration.interfaces.SystemObject[])
   */
  public void update(MutableSet arg0, SystemObject[] arg1, SystemObject[] arg2)
  {
    // TODO Auto-generated method stub
    try
    {
      _debug.fine("update Menge: " + arg0.getName());

      if (arg0.getName().equals("SystemKalenderEintrge"))
      {

        if (arg1 != null)
        {
          List<SystemObject> list = new ArrayList<SystemObject>();

          for (int i = 0; i < arg1.length; i++)
          {

            SystemObject so = arg1[i];

            _debug.fine(so.getPid());

            _debug.fine("added - " + arg1[i]);

            list.add(so);

          }

          _debug.fine("Anmelden auf " + list.size() + " Daten!!!");

          OBJECT_ATG = "atg.systemKalenderEintrag";
          OBJECT_ASP = "asp.parameterSoll";

          _attributgruppe = _connection.getDataModel().getAttributeGroup(OBJECT_ATG);
          _aspekt = _connection.getDataModel().getAspect(OBJECT_ASP);
          _simulationsvariante = 0;
          _datenbeschreibung = new DataDescription(_attributgruppe, _aspekt, _simulationsvariante);
          subscribeReceiver(list);
        }

        if (arg2 != null)
        {
          for (int i = 0; i < arg2.length; i++)
          {
            SystemObject so = arg2[i];

            _debug.fine(so.getPid());

            
            String pidToDel = so.getPid().toLowerCase();
            
            if (pidToDel != null && getSkeList().containsKey(pidToDel))
            {

              getSkeList().remove(pidToDel);
              _debug.fine(pidToDel + " : Objekt geloescht!");

            }
            else
            {
              _debug.fine(pidToDel + " : Objekt nicht verhanden!");
            }

            _debug.fine("removed - " + arg2[i]);

          }

        }

      }
      else if (arg0.getName().equals("Ereignisse"))
      {
        if (arg1 != null)
        {
          List<SystemObject> list = new ArrayList<SystemObject>();

          for (int i = 0; i < arg1.length; i++)
          {

            SystemObject so = arg1[i];

            _debug.fine(so.getPid());

            _debug.fine("added - " + arg1[i]);

            list.add(so);

          }

          OBJECT_ATG = "atg.ereignisParameter";
          OBJECT_ASP = "asp.parameterSoll";

          _attributgruppe = _connection.getDataModel().getAttributeGroup(OBJECT_ATG);
          _aspekt = _connection.getDataModel().getAspect(OBJECT_ASP);
          _simulationsvariante = 0;
          _datenbeschreibung = new DataDescription(_attributgruppe, _aspekt, _simulationsvariante);
          subscribeReceiver(list);
        }

        if (arg2 != null)
        {
          for (int i = 0; i < arg2.length; i++)
          {
            SystemObject so = arg2[i];

            _debug.fine(so.getPid());

            if (getEvtList().containsKey(so.getPid()))
            {
              Ereignis evt = getEvtList().get(so.getPid());
              evt.setErinnerungsFunktion(false, false);

              getEvtList().remove(so.getPid());
              _debug.fine(so.getPid() + " : Objekt geloescht!");

            }
            else
            {

              _debug.fine(so.getPid() + " : Objekt nicht verhanden!");

            }

            _debug.fine("removed - " + arg2[i]);

          }

        }

      }

    }
    catch (RuntimeException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  /**
   * Abstrakte Methode, die implementiert werden muss: bernimmt das tatschliche Senden der Daten
   */
  public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
  {
  }

  /**
   * Abstrakte Methode, die implementiert werden muss: Zeigt, ob die Sendesteuerung erwnscht ist.
   */
  public boolean isRequestSupported(SystemObject so, DataDescription dd)
  {
    return true;
  }

  /**
   * Programmeinstieg.<br>
   * 
   * @param arguments
   *          Kommandozeilenargumente
   */
  public static void main(String[] arguments)
  {
    StandardApplicationRunner.run(new EreignisKalender(), arguments);
  }

  /**
   * Getter fuer die Liste der SystemKalenderEintraege
   * 
   * @return Liste der SystemKalenderEintraege
   * @uml.property name="skeList"
   */
  public static Map<String, SystemkalenderEintrag> getSkeList()
  {
    return skeList;
  }

  /**
   * Setter fuer die Liste der SystemKalenderEintraege
   * 
   * @param Liste
   *          der SystemKalenderEintraege
   * @uml.property name="skeList"
   */
  public static void setSkeList(Map<String, SystemkalenderEintrag> skeList)
  {
    EreignisKalender.skeList = skeList;
  }

  /**
   * Getter fuer die Liste der Ereignisse
   * 
   * @return Liste der Ereignisse
   * @uml.property name="evtList"
   */
  public static Map<String, Ereignis> getEvtList()
  {
    return evtList;
  }

  /**
   * Setter fuer die Liste der Ereignisse
   * 
   * @param evtList
   *          Liste der Ereignisse
   * @uml.property name="evtList"
   */
  public static void setEvtList(Map<String, Ereignis> evtList)
  {
    EreignisKalender.evtList = evtList;
  }

  /**
   * Getter fuer die Verbindung zum DAV
   * 
   * @return Verbindung zum DAV
   * @uml.property name="connection"
   */
  public static ClientDavInterface getConnection()
  {
    return _connection;
  }

  /**
   * Setter fuer die Verbindung zum DAV
   * 
   * @param connection
   *          Verbindung zum DAV
   * @uml.property name="connection"
   */
  public static void setConnection(ClientDavInterface connection)
  {
    _connection = connection;
  }

  public static Map<String, EreignisTyp> getEvtTypList()
  {
    return evtTypList;
  }

  public static void setEvtTypList(Map<String, EreignisTyp> evtTypList)
  {
    EreignisKalender.evtTypList = evtTypList;
  }

  public void parseArbeiterSke()
  {

    try
    {
      // Parsen!!!
      cntParse = 0;

      int cntRekursiv = 0;

      _debug.fine("parseWorker: " + parseList.size());

      while (parseList.size() > 0)
      {

        if (cntRekursiv > 20)
        {
          _debug.fine("Maximale Rekursionstiefe erreicht -> Abbruch des Parsens!!!");
          break;
        }

        _debug.fine("Rekursionstiefe: " + cntRekursiv);

        List<String> delList = new ArrayList<String>();

        Parser parser = new Parser();

        for (Map.Entry<String, String[]> me : parseList.entrySet())
        {

          if (parser.parseSystemkalenderEintrag(me.getKey(), me.getValue()[0], me.getValue()[1]))
          {

            delList.add(me.getKey());

          }
          else
          {
             //_debug.config(me.getKey() + " konnte nicht geparst werden");
          }

          cntParse++;

          synchronized (this)
          {
            notify();
          }

        }

        for (String s : delList)
        {

          parseList.remove(s);

        }

        cntRekursiv++;

      }

      if (_inInit)
      {

        synchronized (this)
        {

          while ((cntParse < skeList.size()))
          {

            this.wait();
          }
        }
        _debug.fine("Alle SystemKalenderEintraege geparst!");
      }
      else
        _debug.fine("SystemKalenderEintrag geparst!");
    }
    catch (InterruptedException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    catch (Exception e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }

  }

  public static Map<String, Boolean[]> getGueltigkeitList()
  {
    return gueltigkeitList;
  }

  public static void setGueltigkeitList(Map<String, Boolean[]> gueltigkeitList)
  {
    EreignisKalender.gueltigkeitList = gueltigkeitList;
  }

}
