/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.ereigniskal.ereigniskal.ereignis;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.ereigniskal.ereigniskal.EreignisKalender;
import de.bsvrz.vew.ereigniskal.ereigniskal.erinnerungsfunktion.ErinnerungsFunktion;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderEintrag;

/**
 * Die Klasse erzeugt Ereignisse. Die Berechnung der Zeitpunkte mit Wildcards bezieht sich noch auf das aktuelle
 * Kalenderjahr. Die Vorgehensweise knnte dahingehend sein, dass beim Jahrewechsel die Zustandswechsel neu berechnet
 * werden. Muss also noch geklrt werden *
 * 
 * @version $Revision: 1.6 $ / $Date: 2015/06/08 15:40:18 $ / ($Author: Pittner $)
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 */
public class Ereignis
{
  /**
   * Die Pid des Ereignisses
   * 
   * @uml.property name="pid"
   */
  private String pid;

  /**
   * Referenz auf den SystemKalenderEintrag
   * 
   * @uml.property name="skeReferenz"
   */
  private String skeReferenz;

  /**
   * Prioritaet des Ereignistyps (EreignisEigenschaften)
   */
  private String ereignisTyp;

  /**
   * Prioritt des Ereignistyps (EreignisParameter)
   */
  private Long ereignisTypPrioritaet;

  /**
   * Ein SystemObjekt
   */
  private SystemObject systemObjekt;

  /**
   * Eine Attributgruppe
   */
  private AttributeGroup atg;

  /**
   * Instanz eines SystemKalenderEintrag
   */
  private SystemkalenderEintrag ske = null;

  /**
   * Instanz des Weckers
   */
  private ErinnerungsFunktion rs = null;

  /**
   * Die Liste der Zeiten der Zustandswechsel fuer die zeitliche Gueltigkeit
   */
  private ListeZustandsWechsel listeZustandsWechselZeit = new ListeZustandsWechsel();


  /**
   * Die Liste der Zeiten der Zustandswechsel fuer die verkehrliche Gueltigkeit
   */
  private ListeZustandsWechsel listeZustandsWechselVerkehr = new ListeZustandsWechsel();


  public int lenVerkehrlicheGueltigkeit = 0;

  public Long anf[];
  public Long end[];

  public Integer bezAnf[];
  public Integer bezEnd[];

  /**
   * Liste der Objektnamen fuer die raeumliche Gueltigkeit
   */
  private List<String> raeumlicheGueltigkeit = new ArrayList<String>();

  /**
   * Der Debugger, stellt verschiedene Protokollierungsfunktionen zur Verfgung
   */
  private Debug _debug;
  

  /**
   * Konstruktor, fuer die Erzeugung mit skeReferenz
   * 
   * @param pid
   *          Die Pid des Ereignisses
   * @param skeReferenz
   *          Referenz auf den SystemKalenderEintrag
   */
  public Ereignis(String pid, String skeReferenz)
  {

    this.pid = pid;  

    _debug = Debug.getLogger();

    if (skeReferenz.length() > 0)
    {
      this.skeReferenz = skeReferenz;

      ske = EreignisKalender.getSkeList().get(skeReferenz);

      if (ske != null)
      {

        listeZustandsWechselZeit.setListeZustandsWechsel(ske.getListeZustandsWechsel());

      }
    }
    else
    {
      listeZustandsWechselZeit.setListeZustandsWechsel(getListeZustandsWechselZeit().getListeZustandsWechsel());
    }

    if (holeConfigData())
    {
      // _debug.config(pid + ": Instanziierung erfolgreich!");
    }
    else{
      _debug.fine(pid + ": Instanziierung fehlgeschlagen!");
    }

  }

  /**
   * Konstruktor, fuer die Erzeugung mit zeitlicherGueltigkeit, d.h. die Definition ueber den Ske wird nicht beachetet
   * 
   * @param pid
   *          Die Pid des Ereignisses
   */
  public Ereignis(String pid)
  {

    this.pid = pid;  

    _debug = Debug.getLogger();

    if (holeConfigData())
    {
      // _debug.config(pid + ": Instanziierung erfolgreich!");
    }
    else{
      _debug.fine(pid + ": Instanziierung fehlgeschlagen!");
    }

  }

  /**
   * Holt die Konfiguratiuonsdaten der Ereignisse bzw. die daraus resultierenden parametrierenden Daten der
   * Ereignistypen
   * 
   * @return true, wenn EreignisTypReferenz erfolgreich geholt werden konnte
   */
  private boolean holeConfigData()
  {

    systemObjekt = EreignisKalender.getConnection().getDataModel().getObject(pid);

    atg = systemObjekt.getDataModel().getAttributeGroup("atg.ereignisEigenschaften");

    Data data = null;

    try
    {
      data = systemObjekt.getConfigurationData(atg);

      ereignisTyp = data.getItem("EreignisTypReferenz").asTextValue().getValueText();

      if (EreignisKalender.getEvtTypList().containsKey(ereignisTyp))
      {

        ereignisTypPrioritaet = EreignisKalender.getEvtTypList().get(ereignisTyp).getPrioritaet();

      }
      else
      {

        _debug.fine(ereignisTyp + " : TypReferenz nicht in evtTypList vorhanden!");

      }

    }
    catch (Exception e)
    {
      // TODO: handle exception
      _debug.fine(ereignisTyp + " : TypReferenz nicht vorhanden!");
    }

    if (ereignisTyp != null)
      if (ereignisTyp.length() > 0 && ereignisTypPrioritaet != null)
        return true;

    return false;
  }

  /**
   * Getter fuer die Pid des Ereignisses
   * 
   * @return String Pid des Ereignisses
   * @uml.property name="pid"
   */
  public String getPid()
  {

    return pid;

  }

  /**
   * Getter fuer die SkeReferenz
   * 
   * @return String die SkeReferenz
   * @uml.property name="skeReferenz"
   */
  public String getSkeReferenz()
  {

    return skeReferenz;
  }

  /**
   * Getter fuer die Referenz auf den EreignisTyp
   * 
   * @return String die EreignisTypReferenz
   * 
   */
  public String getTypReferenz()
  {

    return ereignisTyp;
  }


  /**
   * Getter fuer die raeumliche Gueltigkeit des Ereignisses
   * 
   * @return SortedMap<Long, Boolean> Sortierte Liste der Zustandswechsel
   * @uml.property name="raeumlicheGueltigkeit"
   */
  public List<String> getRaeumlicheGueltigkeit()
  {

    if (raeumlicheGueltigkeit != null)
      return raeumlicheGueltigkeit;

    return null;

  }

  /**
   * Setter fuer die raeumliche Gueltigkeit des Ereignisses
   * 
   * @return SortedMap<Long, Boolean> Sortierte Liste der Zustandswechsel
   */
  public void setRaeumlicheGueltigkeit(String s)
  {

    raeumlicheGueltigkeit.add(s);

  }

  /**
   * Getter fuer ein Wecker-Objekt
   * 
   * @return ReminderService das Wecker-Objekt
   */
  public ErinnerungsFunktion getErinnerungsFunktion()
  {

    return rs;

  }

  /**
   * setter fuer ein Wecker-Objekt
   * 
   * @param f
   *          wenn true, wird ein Wecker gestellt wenn false geht der Wecker gleich los
   */
  public void setErinnerungsFunktion(Boolean f)
  {

    if (f)
      rs = new ErinnerungsFunktion(this, true);
    else
      rs = new ErinnerungsFunktion(this, false);

  }

  /**
   * setter fuer ein Wecker-Objekt
   * 
   * @param f
   *          wenn true, wird ein Wecker gestellt wenn false geht der Wecker gleich los
   */
  public void setErinnerungsFunktion(Boolean b, Boolean c)
  {
    
    if (b)
      rs = new ErinnerungsFunktion(this, true, c);
    else
      rs = new ErinnerungsFunktion(this, false, c);
    
  }
  
  /**
   * setter fuer ein Wecker-Objekt
   * 
   */
  public void setErinnerungsFunktion()
  {   
      rs = new ErinnerungsFunktion(this);
  }
   

  /**
   * setter fuer ein Wecker-Objekt
   * 
   * @param f
   *          wenn true, wird ein Wecker gestellt wenn false geht der Wecker gleich los
   */
  public void setErinnerungsFunktion(Long von, Long bis)
  {

    rs = new ErinnerungsFunktion(this, von, bis);

  }

  public ListeZustandsWechsel getListeZustandsWechselZeit()
  {
    return listeZustandsWechselZeit;
  }

  public void setListeZustandsWechselZeit(ListeZustandsWechsel listeZustandsWechselZeit)
  {
    this.listeZustandsWechselZeit = listeZustandsWechselZeit;
  }

  public ListeZustandsWechsel getListeZustandsWechselVerkehr()
  {
    return listeZustandsWechselVerkehr;
  }

  public void setListeZustandsWechselVerkehr(ListeZustandsWechsel listeZustandsWechselVerkehr)
  {
    this.listeZustandsWechselVerkehr = listeZustandsWechselVerkehr;
  }

  public void setLaengeVerkehrlicheGueltigkeit(int len, boolean init)
  {

    lenVerkehrlicheGueltigkeit = len;

    if (init){
    
    anf = new Long[len];
    end = new Long[len];
    bezAnf = new Integer[len];
    bezEnd = new Integer[len];
    
    }

  }
  
  public  ResultData berechneDatenEreignisZustand(Long zeit)
  {
    ListeZustandsWechsel lzwZ = new ListeZustandsWechsel();

    ListeZustandsWechsel lzwV = new ListeZustandsWechsel();
    
    DataModel datenmodell = EreignisKalender.getConnection().getDataModel();
    
    datenmodell = EreignisKalender.getConnection().getDataModel();

    AttributeGroup atgSend = datenmodell.getAttributeGroup("atg.ereignis");

    Aspect aspSend = datenmodell.getAspect("asp.zustand");

    Data dataSend = EreignisKalender.getConnection().createData(atgSend);

    Boolean flagZeitlich;
    Boolean flagVerkehrlich;

    //dataSend.getTextValue("Attributnderung").setText("Ja");

    lzwZ.setListeZustandsWechsel(this.getListeZustandsWechselZeit().getListeZustandsWechsel());
    lzwV.setListeZustandsWechsel(this.getListeZustandsWechselVerkehr().getListeZustandsWechsel());
     
    // Eintrag ist in beiden Listen vorhanden
    if (this.getListeZustandsWechselZeit().getListeZustandsWechsel().containsKey(zeit)
        && this.getListeZustandsWechselVerkehr().getListeZustandsWechsel().containsKey(zeit))
    {
      flagZeitlich = this.getListeZustandsWechselZeit().getListeZustandsWechsel().get(zeit);

      flagVerkehrlich = this.getListeZustandsWechselVerkehr().getListeZustandsWechsel().get(zeit);

      // Eintrag nur in der zeitl. Gueltigkeitsliste
    }
    else if (this.getListeZustandsWechselZeit().getListeZustandsWechsel().containsKey(zeit))
    {
      flagZeitlich = this.getListeZustandsWechselZeit().getListeZustandsWechsel().get(zeit);

      // Hole den letzten Eintrag verkehrl. Gueltigkeit
      Map.Entry<Long, Boolean> me = lzwV.berechneLetztenZustandsWechsel(zeit);

      if (me != null)
      {

        flagVerkehrlich = me.getValue();

      }
      else
      {

        flagVerkehrlich = false;

      }
      Date d = new Date();
      d.setTime(zeit);

      // Eintrag nur in der verkehrl. Gueltigkeitsliste
    }
    else if (this.getListeZustandsWechselVerkehr().getListeZustandsWechsel().containsKey(zeit))
    {
      flagVerkehrlich = this.getListeZustandsWechselVerkehr().getListeZustandsWechsel().get(zeit);

      // Hole den letzten Eintrag zeitl. Gueltigkeit
      Map.Entry<Long, Boolean> me = lzwZ.berechneLetztenZustandsWechsel(zeit);

      if (me != null)
      {

        flagZeitlich = me.getValue();

      }
      else
      {

        flagZeitlich = false;

      }

      Date d = new Date();
      d.setTime(zeit);
      
      // Kein Eintrag in beiden Listen -> Informieren ueber Zustand der Ereignisse
    }
    else
    {
      SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
      
      Map.Entry<Long, Boolean> mez = lzwZ.berechneLetztenZustandsWechsel(zeit);
      if (mez != null){
        flagZeitlich = mez.getValue();
      }
      else
        flagZeitlich = false;

      Map.Entry<Long, Boolean> mev = lzwV.berechneLetztenZustandsWechsel(zeit);
            
      if (mev != null){
        flagVerkehrlich = mev.getValue();
      }
      else
        flagVerkehrlich = false;

      
      //dataSend.getTextValue("Attributnderung").setText("Nein");

    }

    if (flagZeitlich)
      dataSend.getTextValue("zeitlichGltig").setText("Ja");
    else
      dataSend.getTextValue("zeitlichGltig").setText("Nein");

    if (flagVerkehrlich)
      dataSend.getTextValue("verkehrlichGltig").setText("Ja");
    else
      dataSend.getTextValue("verkehrlichGltig").setText("Nein");

    Map.Entry<Long, Boolean> ez = lzwZ.berechneNaechstenZustandsWechsel(zeit + 1);

    Map.Entry<Long, Boolean> ev = lzwV.berechneNaechstenZustandsWechsel(zeit + 1);
    
    Long l = 0L;

    if (ez != null && ev != null)
    {

      if (ez.getKey() > ev.getKey())

        // ... dann ist der nechste Zustandswechsel auf jeden Fall
        // von verkehrlicher Gueltigkeit
        l = ev.getKey();

      else if (ev.getKey() > ez.getKey())

        // ... dann ist der nechste Zustandswechsel auf jeden Fall
        // von zeitlicher Gueltigkeit
        l = ez.getKey();

      else
      {
        // ... dann ist es egal!
        l = ez.getKey();

      }
      
      Calendar cal = new GregorianCalendar();
      cal.setTimeInMillis(l);
    
      if ((cal.get(Calendar.MILLISECOND) == 999))
        l+=1;

      dataSend.getTimeValue("Zeitpunkt").setMillis(l);
      
    }
    else if (ez == null && ev != null)
    {

      l = ev.getKey();
      
      Calendar cal = new GregorianCalendar();
      cal.setTimeInMillis(l);
    
      if ((cal.get(Calendar.MILLISECOND) == 999))
        l+=1;

      dataSend.getTimeValue("Zeitpunkt").setMillis(l);
 
    }
    else if (ev == null && ez != null)
    {

      l = ez.getKey();

      Calendar cal = new GregorianCalendar();
      cal.setTimeInMillis(l);
    
      if ((cal.get(Calendar.MILLISECOND) == 999))
        l+=1;

      dataSend.getTimeValue("Zeitpunkt").setMillis(l);
     
    }
    else
    {

      //_debug.fine(this.getPid() + " : Es gibt keinen naechsten Zustandwechsel!");

      dataSend.getTimeValue("Zeitpunkt").setMillis(zeit);

    }
    
    
    
    Boolean[] gueltigkeiten = new Boolean[2]; 
    
    if (EreignisKalender.getGueltigkeitList().containsKey(pid)){

      Map<String, Boolean[]> list = EreignisKalender.getGueltigkeitList();
      
      gueltigkeiten = list.get(pid);
      
      if (gueltigkeiten[0] != flagZeitlich || gueltigkeiten[1] != flagVerkehrlich)
        dataSend.getTextValue("Attributnderung").setText("Ja");
      else
        dataSend.getTextValue("Attributnderung").setText("Nein");
      
      EreignisKalender.getGueltigkeitList().remove(pid);
      
    }
    else
      dataSend.getTextValue("Attributnderung").setText("Nein");
    
    gueltigkeiten[0] = flagZeitlich;
    gueltigkeiten[1] = flagVerkehrlich;
    
    EreignisKalender.getGueltigkeitList().put(pid, gueltigkeiten);
    
    
    DataDescription ddSend = new DataDescription(atgSend, aspSend);

    SystemObject sysObj = datenmodell.getObject(this.getPid());

    ResultData resultData = new ResultData(sysObj, ddSend, zeit, dataSend);
    
    return resultData;

  }
  
  public  ResultData setzteDatenEreignisZustand(Boolean b)
  {    
    DataModel datenmodell = EreignisKalender.getConnection().getDataModel();
    
    datenmodell = EreignisKalender.getConnection().getDataModel();

    AttributeGroup atgSend = datenmodell.getAttributeGroup("atg.ereignis");

    Aspect aspSend = datenmodell.getAspect("asp.zustand");

    Data dataSend = EreignisKalender.getConnection().createData(atgSend);

  
    if (b){
    
    dataSend.getTextValue("zeitlichGltig").setText("Ja");
    dataSend.getTextValue("verkehrlichGltig").setText("Ja");
    }else{
      dataSend.getTextValue("zeitlichGltig").setText("Nein");
      dataSend.getTextValue("verkehrlichGltig").setText("Nein");
    }
      
    dataSend.getTextValue("Attributnderung").setText("Nein");

    dataSend.getTimeValue("Zeitpunkt").setMillis(new Date().getTime());
    
    DataDescription ddSend = new DataDescription(atgSend, aspSend);

    SystemObject sysObj = datenmodell.getObject(this.getPid());

    ResultData resultData = new ResultData(sysObj, ddSend, new Date().getTime(), dataSend);
    
    return resultData;

  }
  
  
  public SystemkalenderEintrag getSke()
  {
    return ske;
  }

  public void setSke(SystemkalenderEintrag ske)
  {
    this.ske = ske;
  }

}
